/*
 BarsStress-8Bit v1.00
 Mikhail Svarichevsky http://3.14.by/
 Licensed under Creative Commons Attribution 3.0 Unported License.
 
 This is generic stress test for 8-bit microcontrollers.
 Peripherals here are set up for Arduino. 
 
 It is presumably able to distinguish between sram, flash, code flow and math (multiplication) errors. 
 
 On the display second line lookx like this:
 RUN- XX 12345678
 Where XX test itteration number (just to see it's working)
 12 - checksum for SRAM test, must be 12 if it's ok
 34 - checksum for flash test, must be 34 if it's ok
 56 - checksum for multiplication test, must be 56
 78 - checksum for program flow test (conditional jumps), must be 78
 
 LED blinks on each itteration. 
 If error is detected, led will blink 1 time for sram error, 2 times for flash error, 3 times for multiplication error and 4 times for program flow error.
 Then it will wait 500 milliseconds till next itteration. 
 
 Don't forget, that if you're overclocking - your delays would be incorrect (including display code), and in case of Arduino probably must be corrected in boards.txt.
 
 You can compile this program with full optimization, provided that your compiler don't ignore volatile keyword and places const variables in flash memory. 
 
 PORTING TO OTHER 8-BIT MICROCONTROLLERS's
 
 In the simplest case you need to disable LCD display, implement digitalWrite function which will blink any led on your dev.board, and you probably will need main() function, which should call setup first, then loop() in an endless loop.
 If your microcontrollers lacks hardware multiplication, I would suggest to lower repeat count in run_mul_test down to 35 (so that execution time of all tests is comparable).
 
 Porting to 16-bit and 32-bit microcontrollers is impossible, they would  require completely redesigned math.
 */

#include <LiquidCrystal.h>

LiquidCrystal lcd(12, 11, 5, 4, 3, 2);
int led = 13;//Arduino UNO orange led

//Sample data for reading/writing
volatile unsigned char value1=1;
volatile unsigned char value2=2;
volatile unsigned char value3=3;
volatile unsigned char value4=4;
const unsigned char svalue1=129;
const unsigned char svalue2=1;
const unsigned char svalue3=2;
const unsigned char svalue4=107;

unsigned char sram_check=0x12;
unsigned char flash_check=0x34;
unsigned char mul_check=0x56;
unsigned char flow_check=0x78;

char tmp[32];
unsigned char itteration=0;

unsigned char error_detected = 0;

void setup() {
  lcd.begin(16, 2);
  lcd.print("BarsStress-8Bit");
  pinMode(led, OUTPUT);     
}

void run_flow_test()
{
  for(int repeat=0;repeat<250;repeat++)//We are aiming at ~10 benches per second
  {
    //Program flow check
    //Magic loop which after 93 itterations yields same value
    for(unsigned char i=0;i<93;i++)
    {
      flow_check=(flow_check<<1) + (flow_check>>7) + 25;
      if(flow_check&8)
        flow_check^=4;
      else
        flow_check^=16;
      flow_check=flow_check ^ 192 + 1;
    }
  }
}

void run_mul_test()
{
  for(int repeat=0;repeat<350;repeat++)//We are aiming at ~10 benches per second
  {
    //Multiplication check
    for(unsigned char i=0;i<64;i++)
    {
      mul_check=mul_check*(mul_check-1);
      mul_check=mul_check*(mul_check+1)+2;
    }
  }
}

void run_flash_test()
{
  for(int repeat=0;repeat<1000;repeat++)//We are aiming at ~10 benches per second
  {
    //flash_check
    for(unsigned char i=0;i<16;i++)
    {
      flash_check=(flash_check ^ svalue1) + svalue2;
      flash_check=(flash_check<<((svalue3+flash_check)&7)) + (flash_check>>((svalue3+flash_check)&7)) + svalue4;
    }
  }
}

void run_sram_test()
{
  for(int repeat=0;repeat<10;repeat++)//We are aiming at ~10 benches per second
  {
    //SRAM check
    for(int i=0;i<2405;i++)
    {
      value1=(value1+value2+value3+value4+sram_check)&1;
      value2=(value2+value1+value3+value4+sram_check)&1+1;
      value3=(value3+value1+value2+value4+sram_check)&1+2;
      value4=(value4+value1+value2+value3+sram_check)&1+3;
      sram_check=(sram_check<<1)+(sram_check>>7)+value1+value2+value3+value4;
    }
  }
}

void detect_errors()
{
  //Detect type error for LED indication
  if(sram_check!=0x12)
    error_detected=1;
  if(flash_check!=0x34)
    error_detected=2;
  if(mul_check!=0x56)
    error_detected=3;
  if(flow_check!=0x78)
    error_detected=4;


  if(error_detected)
  {
    //Signalize detected error code by blinking led
    for(unsigned char i=0;i<error_detected;i++)
    {
      digitalWrite(led, 1);
      delay(100);
      digitalWrite(led, 0);
      delay(200);
    }
    digitalWrite(led, 0);
    delay(500);
  }
  else
    digitalWrite(led, itteration&1);//Blinking means all works fine
}

void refresh_lcd_display()
{
  lcd.setCursor(0, 1);
  lcd.print("RUN");

  if(millis()%250<125)//This is designed for 64Mhz clock
    lcd.print("\xff ");
  else
    lcd.print("\xb0 ");


  sprintf(tmp, "%.2X %.2X%.2X%.2X%.2X ", itteration, sram_check, flash_check, mul_check, flow_check);
  lcd.print(tmp);
}

void loop() {
  itteration+=1;

  run_flow_test();
  run_mul_test();
  run_flash_test();
  run_sram_test();

  //  if(rand()%20==0)flow_check++; Error indication test
  detect_errors();

  refresh_lcd_display();
}


